@@ -167,7 +167,7 @@ static int write_command(iso15693_t* iso15693, int command, u_int8_t* buffer, in
 {
   rfid_reader_t* driver = iso15693->reader;
 
-  char* request = (char*)malloc(buffer_size + ISO15693_COMMAND_OVERHEAD);
+  u_int8_t* request = (u_int8_t*)malloc(buffer_size + ISO15693_COMMAND_OVERHEAD);
   short request_length = buffer_size + ISO15693_COMMAND_OVERHEAD;
   int retval;
 
@@ -216,7 +216,7 @@ static int read_response(iso15693_t* iso15693, u_int8_t** bufferp, int* buffer_s
   }
 
   if(buffer[ISO15693_FLAGS_INDEX] & ISO15693_RESPONSE_FLAG_ERROR) {
-    iso15693->error = error_map[buffer[ISO15693_ERROR_INDEX]];
+    iso15693->error = error_map[buffer[ISO15693_ERROR_INDEX] & 0x1F];
     free(buffer);
     RFID_ERROR_STACK(iso15693->verbose);
     return -1;
@@ -549,8 +549,8 @@ static int read_multiple_blocks(iso15693_t* iso15693, const rfid_transponder_t*
   }
 
   {
-    index = 0;
     int blocks_index = 0;
+    index = 0;
 
     for(blocks_index = 0; blocks_index < blocks_length; blocks_index++) {
       if(want_security_status) {
@@ -830,7 +830,7 @@ int iso15693_lock_dsfid(iso15693_t* iso15693, const u_int8_t* uid)
 
 #ifndef ISO15693_GET_SYSTEM_INFORMATION_FILE
 #define ISO15693_GET_SYSTEM_INFORMATION_FILE "/var/lib/rfid/system_information.db"
-#endif // ISO15693_GET_SYSTEM_INFORMATION_FILE
+#endif /* ISO15693_GET_SYSTEM_INFORMATION_FILE */
 
 int iso15693_get_system_information(iso15693_t* iso15693, rfid_transponder_t* transponder)
 {
@@ -1089,7 +1089,7 @@ int iso15693_get_multiple_block_security_status(iso15693_t* iso15693, const u_in
   return 0;
 }
 
-int iso15693_response_size(rfid_reader_t* reader, char* request, int request_size, int* response_size, const rfid_transponder_t* transponder)
+int iso15693_response_size(rfid_reader_t* reader, u_int8_t* request, int request_size, int* response_size, const rfid_transponder_t* transponder)
 {
   iso15693_t* iso15693 = (iso15693_t*)reader->private;
 
@@ -1219,9 +1219,10 @@ static int driver_load(iso15693_t* iso15693, const char* drivers)
 static int drv_probe(struct rfid_reader* reader, const char* drivers)
 {
   iso15693_t* iso15693 = (iso15693_t*)reader->private;
-  iso15693->io = reader->io;
   int retval;
 
+  iso15693->io = reader->io;
+
   strcpy(reader->drivers, drivers);
   if(driver_load(iso15693, drivers) < 0) {
     RFID_ERROR_STACK(iso15693->verbose);
@@ -1312,7 +1313,7 @@ static int drv_error(struct rfid_reader* reader)
 static int drv_read(struct rfid_reader* reader, int block_start, int number_of_blocks, rfid_transponder_t* transponder)
 {
   iso15693_t* iso15693 = (iso15693_t*)reader->private;
-  rfid_block_t blocks[number_of_blocks];
+  rfid_block_t* blocks;
   int i;
   u_int8_t* pointer;
   int retval;
@@ -1329,6 +1330,8 @@ static int drv_read(struct rfid_reader* reader, int block_start, int number_of_b
 
   pointer = transponder->data + block_start * transponder->bytes_per_block;
   
+  blocks = (rfid_block_t*)malloc(sizeof(rfid_block_t) * number_of_blocks);
+  
   for(i = 0; i < number_of_blocks; i++) {
     blocks[i].data = pointer;
     pointer += transponder->bytes_per_block;
@@ -1346,6 +1349,8 @@ static int drv_read(struct rfid_reader* reader, int block_start, int number_of_b
     if(iso15693->verbose == 1) fprintf(stderr, "R");
   }
 
+  free(blocks);
+
   return retval;
 }
 
