@@ -1245,7 +1245,7 @@ int iso15693_get_multiple_block_security_status(iso15693_t* iso15693, const u_in
  * transponder argument is only necessary when the response size depends
  * on the block size of the transponder.
  */
-int iso15693_response_size(rfid_reader_t* reader, char* request, int request_size, int* response_size, const rfid_transponder_t* transponder);
+int iso15693_response_size(rfid_reader_t* reader, u_int8_t* request, int request_size, int* response_size, const rfid_transponder_t* transponder);
 
 /*
  * Set CRC16 at the end of a ISO-15693 frame (last two bytes).
