@@ -1220,7 +1220,7 @@ static int drv_init(struct rfid_reader* reader, const char* drivers)
 static void drv_end(struct rfid_reader* reader)
 {
   m2xxh_t* m2xxh = m2xxh_from_reader(reader);
-  return m2xxh_end(m2xxh);
+  m2xxh_end(m2xxh);
 }
 
 static char* drv_strerror(struct rfid_reader* reader)
@@ -1466,8 +1466,33 @@ static int drv_iso15693_response(struct rfid_reader* reader, u_int8_t** datap, i
   int response_size = m2xxh->iso15693_response_length < ISO15693_ERROR_FRAME_SIZE ? ISO15693_ERROR_FRAME_SIZE : m2xxh->iso15693_response_length;
   u_int8_t* response = (u_int8_t*)malloc(response_size);
 
-  if(m2xxh->iso15693_response[ISO15693_FLAGS_INDEX] & ISO15693_RESPONSE_FLAG_ERROR)
+  if(m2xxh->iso15693_response[ISO15693_FLAGS_INDEX] & ISO15693_RESPONSE_FLAG_ERROR) {
+    if(!iso15693_crc16_check(m2xxh->iso15693_response, ISO15693_ERROR_FRAME_SIZE) || m2xxh->iso15693_response[ISO15693_FLAGS_INDEX] == 0xFF) {
+      /*
+       * If the transponder left the RFID reader range immediately
+       * after sending the SOF, the buffer returned will be filled
+       * with 0xFF, which is the initialization value of the RFID
+       * reader buffer. From the ISO-15693-3 point of view it looks
+       * like an error. However, it is not a well formed error and
+       * we must detect this case.
+       *
+       * If the buffer is filled with 0xFF we hit the pathological
+       * CRC16 case : 0xFFFF is a valid checksum of 0xFF 0xFF. We
+       * can just rely on the checksum to tell us if the error message
+       * is garbled, we must specificaly take in account the case
+       * where the first byte is 0xFF.
+       *
+       * If the error message is invalid because of the above,
+       * pretend it is a TRANSPONDER_NOT_FOUND error.
+       */
+      m2xxh->iso15693_response[ISO15693_FLAGS_INDEX] = ISO15693_RESPONSE_FLAG_ERROR;
+      m2xxh->iso15693_response[ISO15693_ERROR_INDEX] = NOT_ISO15693_ERROR_TRANSPONDER_NOT_FOUND;
+      iso15693_crc16_set(m2xxh->iso15693_response, ISO15693_ERROR_FRAME_SIZE);
+      
+    }
+    
     m2xxh->iso15693_response_length = ISO15693_ERROR_FRAME_SIZE;
+  }
   
   memcpy(response, m2xxh->iso15693_response, m2xxh->iso15693_response_length);
 
