@@ -94,12 +94,13 @@ typedef struct {
 
 static int rfid_probe(rfid_t* rfid, rfid_probe_t* probe)
 {
-  rfid_reader_t* probers[probe->drivers_length];
+  rfid_reader_t** probers;
   int retval = -1;
   int device_index;
   int driver_index;
   int found = 0;
 
+  probers = (rfid_reader_t**)malloc(probe->drivers_length * sizeof(rfid_reader_t*));;
   probe->readers_length = 0;
 
   memset(probers, '\0', sizeof(rfid_reader_t*) * probe->drivers_length);
@@ -193,6 +194,7 @@ static int rfid_probe(rfid_t* rfid, rfid_probe_t* probe)
       rfid_free(probers[driver_index]);
     }
   }
+  free(probers);
 
   if(retval < 0) {
     /*
@@ -238,17 +240,16 @@ int rfid_init(rfid_t* rfid, const char* driver, const char* device)
 
   rfid_reader_t* readers[8 * 3] = { 0, };
 
-  rfid_probe_t probe = {
-    drivers,
-    drivers_length,
-    devices,
-    devices_length,
-
-    readers,
-    0
-  };
+  rfid_probe_t probe;
   int i;
 
+  probe.drivers = drivers;
+  probe.drivers_length = drivers_length;
+  probe.devices = devices;
+  probe.devices_length = devices_length;
+  probe.readers = readers;
+  probe.readers_length = 0;
+
   if(driver) {
     arg_drivers[0] = driver;
     probe.drivers = arg_drivers;
@@ -425,16 +426,15 @@ static int transponders_diff(rfid_t* rfid, rfid_events_context_t* context, int*
   int cache_index;
 
   /*
-   * Compare the cache with the current transponders until one of the
-   * list is exhausted.
+   * Compare the cache with the current transponders.
    */
   for(current_index = 0, cache_index = 0;
       current_index < current->length || cache_index < cache->length;) {
     int retval;
     /*
-     * -1 => transponders[i] is new on the RFID reader
-     * 1 => cache[i] was removed from the RFID reader
-     * 0 => transponders[i] is still on the RFID reader
+     * -1 => current->list[i] is new on the RFID reader
+     * 1 => cache->list[i] was removed from the RFID reader
+     * 0 => current->list[i] is still on the RFID reader
      */
     int compare;
 
@@ -442,14 +442,16 @@ static int transponders_diff(rfid_t* rfid, rfid_events_context_t* context, int*
       compare = reader->transponder_cmp_f(reader, current->list[current_index], cache->list[cache_index]);
     } else if(current_index < current->length) {
       /*
-       * What's left in current transponders is new because it matches
-       * none of the cached transponders.
+       * If current is longer than cache, what's left in 
+       * current are RFID transponders that just entered
+       * the range of the RFID reader.
        */
       compare = -1;
     } else {
       /*
-       * What's left in transponders cache is deleted because it matches
-       * none of the current transponders.
+       * If cache is longer than current, what's left in cache
+       * describe RFID transponders that left the range of the RFID
+       * reader.
        */
       compare = 1;
     }
@@ -458,47 +460,58 @@ static int transponders_diff(rfid_t* rfid, rfid_events_context_t* context, int*
       (*changedp)++;
       if(rfid->verbose == 1) fprintf(stderr, "N");
       if(block_signals(rfid) < 0) {
+	RFID_ERROR_STACK(rfid->verbose);
 	return RFID_CALLBACK_ERROR;
       }
       /*
-       * Fill structural information about the transponder.
+       * Fill meta information.
        */
       if(reader->transponder_present_f(reader, current->list[current_index]) < 0) {
 	/*
-	 * The state changed, try again.
+	 * The state changed (the RFID transponder probably left the
+	 * range of the RFID reader), try again.
 	 */
 	if(reader->error_f(reader) == RFID_ERROR_TRANSPONDER_NOT_FOUND)
 	  rfid->error = RFID_ERROR_TRY_AGAIN;
 	release_signals(rfid);
+	RFID_ERROR_STACK(rfid->verbose);
 	return RFID_CALLBACK_ERROR;
       }
       /*
-       * Read all data from it.
+       * Read data from the memory of the RFID transponder.
        */
       if(reader->read_f(reader, 0, current->list[current_index]->blocks, current->list[current_index]) < 0) {
 	/*
-	 * The state changed, try again.
+	 * The state changed (the RFID transponder probably left the
+	 * range of the RFID reader), try again.
 	 */
 	if(reader->error_f(reader) == RFID_ERROR_TRANSPONDER_NOT_FOUND)
 	  rfid->error = RFID_ERROR_TRY_AGAIN;
 	release_signals(rfid);
+	RFID_ERROR_STACK(rfid->verbose);
 	return RFID_CALLBACK_ERROR;
       }
 
-      if(release_signals(rfid) < 0)
+      if(release_signals(rfid) < 0) {
+	RFID_ERROR_STACK(rfid->verbose);
 	return RFID_CALLBACK_ERROR;
+      }
 
       if(events_mask & RFID_EVENT_NEW) {
-	if((retval = (*transponder_callback)(rfid, RFID_EVENT_NEW, current->list[current_index], data)) != 0) 
+	if((retval = (*transponder_callback)(rfid, RFID_EVENT_NEW, current->list[current_index], data)) != 0) {
+	  if(retval < 0) RFID_ERROR_STACK(rfid->verbose);
 	  return retval;
+	}
       }
       current_index++;
     } else if(compare > 0) {
       (*changedp)++;
       if(rfid->verbose == 1) fprintf(stderr, "D");
       if(events_mask & RFID_EVENT_DELETE) {
-	if((retval = (*transponder_callback)(rfid, RFID_EVENT_DELETE, cache->list[cache_index], data)) != 0) 
+	if((retval = (*transponder_callback)(rfid, RFID_EVENT_DELETE, cache->list[cache_index], data)) != 0) {
+	  if(retval < 0) RFID_ERROR_STACK(rfid->verbose);
 	  return retval;
+	}
       }
       cache_index++;
     } else if(compare == 0) {
@@ -509,8 +522,10 @@ static int transponders_diff(rfid_t* rfid, rfid_events_context_t* context, int*
       reader->transponder_copy_f(reader, current->list[current_index], cache->list[cache_index]);
       if(rfid->verbose == 1) fprintf(stderr, "A");
       if(events_mask & RFID_EVENT_AGAIN) {
-	if((retval = (*transponder_callback)(rfid, RFID_EVENT_AGAIN, current->list[current_index], data)) != 0) 
+	if((retval = (*transponder_callback)(rfid, RFID_EVENT_AGAIN, current->list[current_index], data)) != 0) {
+	  if(retval < 0) RFID_ERROR_STACK(rfid->verbose);
 	  return retval;
+	}
       }
       current_index++;
       cache_index++;
@@ -626,10 +641,9 @@ static int event_loop(rfid_t* rfid, rfid_events_context_t* context)
     }
 
     /*
-     * Short pause to avoid using all the CPU. More a safety
-     * measure than really usefull since the delay imposed
-     * by the serial line dialog are more than likely to introduce
-     * enough delay anyway.
+     * Short pause to avoid using all the CPU. More a safety measure
+     * than really usefull since the speed of the serial line is more
+     * than likely to introduce enough delay anyway.
      */
     nanosleep(&zzz, 0);
   }
