@@ -33,20 +33,12 @@
 
 #include <signal.h>
 #include <rfid_reader.h>
+#include <rfid_error.h>
 
 /*
  * Values of the options field of the rfid_t structure.
  */
-/*
- * If set, the RFID reader is assumed to respond to the ISO-15695-3
- * protocol.
- */
-#define RFID_OPTION_ISO15693		0x00000002
-/*
- * If set, the frames sent/received to/from the RFID reader
- * are dumped on stderr.
- */
-#define RFID_OPTION_DUMP_FRAMES		0x00000004
+#define RFID_OPTION_FREE_IO		0x00000002
 
 /*
  * Handler holding context for a specific RFID device and used by
@@ -57,22 +49,25 @@ typedef struct {
   int options;			/* Flags controling the behavior of
 				   rfid functions. */
   int error;			/* Errors set by rfid functions. */
-  char* device;			/* Path name of the device connecting
-				   to the RFID reader. */
-  int verbose;			/* Control verbosity level of
-				   functions. */
+  int verbose;			/* Control verbosity level. */
   sigset_t sigset;		/* Saved signal handlers state. */
 } rfid_t;
 
 /*
- * Opens rfid->device that must be a tty and initializes it in raw
- * mode. Probe the presence of a RFID reader and sets the carrier on.
+ * Placeholders for the driver and device arguments of the
+ * rfid_init function.
  */
-int rfid_init(rfid_t* rfid);
+#define RFID_DRIVER_GUESS	0
+#define RFID_DEVICE_GUESS	0
 
 /*
- * Cleanup the context and closes the device used to dialog with
- * the RFID reader.
+ * Load the RFID drivers modules and call their initialization function.
+ */
+int rfid_init(rfid_t* rfid, const char* driver, const char* device);
+
+/*
+ * Call the destructors of the RFID drivers modules and
+ * cleanup internal state.
  */
 void rfid_end(rfid_t* rfid);
 
@@ -92,8 +87,8 @@ void rfid_transponder_free(rfid_t* rfid, rfid_transponder_t* transponder);
  */
 char* rfid_transponder_describe(rfid_t* rfid, rfid_transponder_t* transponder);
 /*
- * Return a null terminated string containing an ascii representation of the
- * unique identifier of the transponder given in argument.
+ * Return a null terminated string containing an ascii representation
+ * of the unique identifier of the transponder given in argument.
  */
 int rfid_transponder_id_get(rfid_t* rfid, const rfid_transponder_t* transponder, char** idp);
 /*
@@ -107,16 +102,33 @@ int rfid_transponder_id_set(rfid_t* rfid, rfid_transponder_t* transponder, char*
  */
 
 /*
- * Valid return values of rfid_read_callback_t functions.
+ * Valid return values of rfid_transponder_callback_t functions.
  */
-#define RFID_CALLBACK_OK	0	/* Everything is fine, keep going. */
-#define RFID_CALLBACK_END	1	/* Everything is fine, end outer loop. */
-#define RFID_CALLBACK_ERROR	-1	/* Something went wrong, end outer loop on error. */
+#define RFID_CALLBACK_OK	0	/* Everything is fine, keep
+					   going. */
+#define RFID_CALLBACK_END	1	/* Everything is fine, end
+					   outer loop. */
+#define RFID_CALLBACK_ERROR	-1	/* Something went wrong, end
+					   outer loop on error. */
 
-#define RFID_EVENT_NEW		0x01	
-#define RFID_EVENT_DELETE	0x02	
-#define RFID_EVENT_AGAIN	0x04
-#define RFID_EVENT_INVENTORY	0x08
+/*
+ * Values of the event argument of the rfid_transponder_callback_t
+ * funtion and of the event_mask argument of the rfid_event_loop 
+ * function.
+ */
+#define RFID_EVENT_NEW		0x01	/* A transponder entered the
+					   range of the RFID
+					   reader. */
+#define RFID_EVENT_DELETE	0x02	/* A transponder left the
+					   range of the RFID
+					   reader. */
+#define RFID_EVENT_AGAIN	0x04    /* A transponder is still in
+					   the range of the RFID
+					   reader. */
+#define RFID_EVENT_INVENTORY	0x08	/* An inventory of all
+					   transponders in the range
+					   of the RFID reader was
+					   completed. */
 #define RFID_EVENT_ALL		0xFF
 #define RFID_EVENT_NONE		0x00
 
@@ -133,7 +145,7 @@ typedef int rfid_transponder_callback_t(rfid_t* rfid, int event, const rfid_tran
 
 /*
  * Called at the end of each rfid_event_loop iteration. The
- * transponders argument is an array of pointers to rfid_transponders_t
+ * transponders argument is an array of pointers to rfid_transponder_t
  * objects that fully describe the RFID transponders including meta
  * information. The data argument is an opaque pointer
  * identical to the data argument of the rfid_event_loop function.
@@ -156,7 +168,7 @@ int rfid_event_loop(rfid_t* rfid, int events_mask, rfid_transponder_callback_t*
 /*
  * Write transponder->data_length bytes from transponder->data to
  * a RFID transponder. If the length of the data written does
- * not fill the available space on the RFID transponder, the
+ * not fill the space available on the RFID transponder, the
  * remaining bytes are set to NULL.
  */
 int rfid_write(rfid_t* rfid, rfid_transponder_t* transponder);
@@ -172,11 +184,6 @@ char* rfid_strerror(rfid_t* rfid);
  */
 int rfid_error(rfid_t* rfid);
 
-/*
- * Set the options to value.
- */
-#define rfid_options(rfid,value) ((rfid)->options = (value))
-
 /*
  * Return the version of the library as a string ("1.0.0" for instance).
  */
