@@ -87,7 +87,7 @@ static int transponder_callback(rfid_t* rfid, int event, const rfid_transponder_
   }
 
   {
-    int retval = system(transponder->data);
+    int retval = system((char*)transponder->data);
     if(commander->verbose)
       fprintf(stderr, "command '%s' returned %d\n", transponder->data, retval);
   }
